/*
 * Copyright (c) 2007 BUSINESS OBJECTS SOFTWARE LIMITED
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *  
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *  
 *     * Neither the name of Business Objects nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *  
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


/*
 * NRecordToJRecordValuePrimitive.java
 * Created: Jan 12, 2007
 * By: Bo Ilic
 */

package org.openquark.cal.internal.machine.g.functions;

import java.util.List;

import org.openquark.cal.compiler.FieldName;
import org.openquark.cal.compiler.QualifiedName;
import org.openquark.cal.internal.machine.g.Executor;
import org.openquark.cal.internal.machine.g.NPrimitiveFunc;
import org.openquark.cal.internal.machine.g.NRecordValue;
import org.openquark.cal.internal.machine.g.NValObject;
import org.openquark.cal.internal.machine.g.Node;
import org.openquark.cal.internal.module.Cal.Core.CAL_Prelude_internal;
import org.openquark.cal.internal.module.Cal.Core.CAL_Record_internal;
import org.openquark.cal.internal.runtime.RecordValueImpl;
import org.openquark.cal.module.Cal.Core.CAL_Prelude;
import org.openquark.cal.runtime.CALExecutorException;
import org.openquark.cal.runtime.RecordValue;


/**
 * Implementation of Prelude.recordToJListPrimitive :: Outputable r => {r} -> JList;
 */
public final class NRecordToJRecordValuePrimitive extends NPrimitiveFunc {

    public static final QualifiedName name = CAL_Record_internal.Functions.recordToJRecordValuePrimitive;
    public static final NRecordToJRecordValuePrimitive instance = new NRecordToJRecordValuePrimitive ();
    
    private boolean implementationChecked = false;
    
    
    private NRecordToJRecordValuePrimitive () {/* Constructor made private for creation control. */}
      
    @Override
    protected int getArity () {
        return 2;
    }
  
    @Override
    protected QualifiedName getName () {
        return name;
    }

    /**
     * Convert an internal record value into the java representation as a Java map.
     */
    @Override
    public Node doEvaluation (Node[] arguments, Executor executor)
            throws CALExecutorException {

        //recordToJRecordValuePrimitive recordDictionary x.
        //the compiler ensures that the 2 record arguments all have the same fields.
        //must iterate in a deterministic order over the field names (as specified by FieldName.CalSourceFormComparator)
        //so that the function is well-defined in the presence of side effects.
        //If f is a field, then recordDictionary.f is the dictionary for use when calling the class method Prelude.output on
        //the value x.f.  
        
        checkImplementation(executor);
        
        // The two arguments in evaluated form will be on top of the stack, in reverse order.
        NRecordValue recordDictionary = (NRecordValue) executor.internalEvaluate(arguments[0]);
        NRecordValue x = (NRecordValue) executor.internalEvaluate(arguments[1]);
        
        List<String> fieldNames = recordDictionary.fieldNames();        
        int nFields = fieldNames.size();
        
        RecordValue resultMap = RecordValueImpl.make();
        for (int i = 0; i < nFields; ++i) {
            
            String fieldName = fieldNames.get(i);
            Node valueDictionaryThunk = recordDictionary.getValue(fieldName);
            Node xValueThunk = x.getValue(fieldName);            
            
            //compute "Prelude.output valueDictionaryThunk xValueThunk"
            //this is just (after inlining Prelude.output d = d"
            //valueDictionaryThunk xValueThunk
            
            Object fieldVal = executor.internalEvaluate(valueDictionaryThunk.apply (xValueThunk)).getValue();
            resultMap.put(FieldName.make(fieldName), fieldVal);
        }
        
        return new NValObject(resultMap);
        
    }

    private void checkImplementation (Executor executor) {
        if(!implementationChecked) {
            if (!isSingleMethodRootClass(executor, CAL_Prelude.TypeClasses.Outputable)) {           
                throw new IllegalStateException("The implementation of " + CAL_Prelude_internal.Functions.recordFromJMapPrimitive.getQualifiedName() + " assumes that the class " + CAL_Prelude.TypeClasses.Outputable.getQualifiedName() + " has no superclasses and only 1 class method.");
            }
            implementationChecked = true;
        }
    }
}
